@@ -1,4 +1,4 @@
-//@line 42 "/home/karma/src/build/build/browser/components/nsBrowserGlue.js"
+//@line 42 "/home/karma/src/build/browser/components/nsBrowserGlue.js"
 
 const Ci = Components.interfaces;
 const Cc = Components.classes;
@@ -59,6 +59,11 @@ function BrowserGlue() {
                                    getService(Ci.nsIObserverService);
   });
 
+  this.__defineGetter__("_distributionCustomizer", function() {
+    delete this._distributionCustomizer;
+    return this._distributionCustomizer = new DistributionCustomizer()
+  });
+
   this._init();
 }
 
@@ -121,6 +126,10 @@ BrowserGlue.prototype = {
         this._observerService.removeObserver(this, "places-init-complete");
         // no longer needed, since history was initialized completely.
         this._observerService.removeObserver(this, "places-database-locked");
+
+        // Now apply distribution customized bookmarks.
+        // This should always run after Places initialization.
+        this._distributionCustomizer.applyBookmarks();
         break;
       case "places-database-locked":
         this._isPlacesDatabaseLocked = true;
@@ -134,6 +143,12 @@ BrowserGlue.prototype = {
           this._archiveBookmarks();
         }
         break;
+      case "distribution-customization-complete":
+        this._observerService
+            .removeObserver(this, "distribution-customization-complete");
+        // Customization has finished, we don't need the customizer anymore.
+        delete this._distributionCustomizer;
+        break;
     }
   }, 
 
@@ -152,6 +167,7 @@ BrowserGlue.prototype = {
     osvr.addObserver(this, "session-save", false);
     osvr.addObserver(this, "places-init-complete", false);
     osvr.addObserver(this, "places-database-locked", false);
+    osvr.addObserver(this, "distribution-customization-complete", false);
   },
 
   // cleanup (called on application shutdown)
@@ -173,8 +189,7 @@ BrowserGlue.prototype = {
   {
     // apply distribution customizations (prefs)
     // other customizations are applied in _onProfileStartup()
-    var distro = new DistributionCustomizer();
-    distro.applyPrefDefaults();
+    this._distributionCustomizer.applyPrefDefaults();
   },
 
   // profile startup handler (contains profile initialization routines)
@@ -193,8 +208,7 @@ BrowserGlue.prototype = {
 
     // apply distribution customizations
     // prefs are applied in _onAppDefaults()
-    var distro = new DistributionCustomizer();
-    distro.applyCustomizations();
+    this._distributionCustomizer.applyCustomizations();
 
     // handle any UI migration
     this._migrateUI();
@@ -383,10 +397,10 @@ BrowserGlue.prototype = {
       return !this._prefs.getBoolPref("browser.EULA.override");
     } catch (e) { }
 
-//@line 427 "/home/karma/src/build/build/browser/components/nsBrowserGlue.js"
+//@line 441 "/home/karma/src/build/browser/components/nsBrowserGlue.js"
     // Non-official builds shouldn't shouldn't show the notification.
     return false;
-//@line 430 "/home/karma/src/build/build/browser/components/nsBrowserGlue.js"
+//@line 444 "/home/karma/src/build/browser/components/nsBrowserGlue.js"
 
     // Look to see if the user has seen the current version or not.
     var currentVersion = this._prefs.getIntPref("browser.rights.version");
@@ -911,7 +925,7 @@ BrowserGlue.prototype = {
     }
   },
 
-//@line 957 "/home/karma/src/build/build/browser/components/nsBrowserGlue.js"
+//@line 971 "/home/karma/src/build/browser/components/nsBrowserGlue.js"
 
   // this returns the most recent non-popup browser window
   getMostRecentBrowserWindow : function ()
@@ -919,7 +933,7 @@ BrowserGlue.prototype = {
     var wm = Cc["@mozilla.org/appshell/window-mediator;1"].
              getService(Components.interfaces.nsIWindowMediator);
 
-//@line 965 "/home/karma/src/build/build/browser/components/nsBrowserGlue.js"
+//@line 979 "/home/karma/src/build/browser/components/nsBrowserGlue.js"
     var win = wm.getMostRecentWindow("navigator:browser", true);
 
     // if we're lucky, this isn't a popup, and we can just return this
@@ -933,7 +947,7 @@ BrowserGlue.prototype = {
           win = nextWin;
       }
     }
-//@line 991 "/home/karma/src/build/build/browser/components/nsBrowserGlue.js"
+//@line 1005 "/home/karma/src/build/browser/components/nsBrowserGlue.js"
 
     return win;
   },
@@ -1015,8 +1029,9 @@ GeolocationPrompt.prototype = {
       var buttons = [{
               label: browserBundle.GetStringFromName("geolocation.shareLocation"),
               accessKey: browserBundle.GetStringFromName("geolocation.shareLocation.accesskey"),
-              callback: function(notification) {                  
-                  if (notification.getElementsByClassName("rememberChoice")[0].checked)
+              callback: function(notification) {
+                  var elements = notification.getElementsByClassName("rememberChoice");
+                  if (elements.length && elements[0].checked)
                       setPagePermission(request.requestingURI, true);
                   request.allow(); 
               },
@@ -1025,7 +1040,8 @@ GeolocationPrompt.prototype = {
               label: browserBundle.GetStringFromName("geolocation.dontShareLocation"),
               accessKey: browserBundle.GetStringFromName("geolocation.dontShareLocation.accesskey"),
               callback: function(notification) {
-                  if (notification.getElementsByClassName("rememberChoice")[0].checked)
+                  var elements = notification.getElementsByClassName("rememberChoice");
+                  if (elements.length && elements[0].checked)
                       setPagePermission(request.requestingURI, false);
                   request.cancel();
               },
@@ -1046,10 +1062,16 @@ GeolocationPrompt.prototype = {
       // bar.
       function geolocation_hacks_to_notification () {
 
-        var checkbox = newBar.ownerDocument.createElementNS(XULNS, "checkbox");
-        checkbox.className = "rememberChoice";
-        checkbox.setAttribute("label", browserBundle.GetStringFromName("geolocation.remember"));
-        newBar.appendChild(checkbox);
+        // Never show a remember checkbox inside the private browsing mode
+        var inPrivateBrowsing = Cc["@mozilla.org/privatebrowsing;1"].
+                                getService(Ci.nsIPrivateBrowsingService).
+                                privateBrowsingEnabled;
+        if (!inPrivateBrowsing) {
+          var checkbox = newBar.ownerDocument.createElementNS(XULNS, "checkbox");
+          checkbox.className = "rememberChoice";
+          checkbox.setAttribute("label", browserBundle.GetStringFromName("geolocation.remember"));
+          newBar.appendChild(checkbox);
+        }
 
         var link = newBar.ownerDocument.createElementNS(XULNS, "label");
         link.className = "text-link";
